(*
 * Title:  Command Line/Environment Information
 * Author: Gregory Elder
 * Last Update (by Gregory Elder):  September 25, 1986
 * Last Update (by Bob Dukelow): January 19, 1987
 * Compiler: Logitech Modula-2/86
 * Description:
 * This module provides argc and argv features similar to C and a procedure
 * for getting the values of environment variables.  ArgC is the count of the
 * number of parameters on the command line.  ArgV is an array of pointers
 * to strings containing the various command line parameters.  GetEnv is
 * a procedure which, when given an enviroment variable, will return the
 * value the variable is set to.
 *
 *)

IMPLEMENTATION MODULE CmdEnv;

FROM SYSTEM IMPORT
	AX, BX, CX, GETREG, SETREG, SWI, RTSVECTOR, ADDRESS;
FROM Storage IMPORT
	ALLOCATE;
FROM ASCII IMPORT
	nul;
FROM Strings IMPORT
	Length, CompareStr;


CONST
	CmdTailAdr  = 80H;
	EnvOffSet   = 2CH;
        EnvSize     = 0FFFEH (* MAX(CARDINAL) - 1 *);

TYPE
	CardPTR = POINTER TO CARDINAL;
	EnvStringPTR = POINTER TO ARRAY[0..EnvSize] OF CHAR;

VAR
	PSP, AnAddress : ADDRESS;
	EnvPTR : EnvStringPTR;
	EnvAdrSegment : CardPTR;

PROCEDURE ParseCmdLine;
VAR
	CmdLength, i, j, start : CARDINAL;
	CmdLine, Sptr : StringPTR;
	CmdAdr : ADDRESS;
BEGIN
	CmdAdr := PSP;
	INC(CmdAdr, CmdTailAdr);		(* Get command tail *)
	CmdLine := CmdAdr;
	i := 0;
	CmdLength := ORD(CmdLine^[i]) - 1;
	INC(i);
	WHILE (i <= CmdLength) AND (CmdLine^[i] = ' ') DO
		INC(i);
		END;
	WHILE (i <= CmdLength) AND (CmdLine^[i] # ' ') DO
		INC(i);
		END;
	ArgV[0] := NIL;
	ArgC := 0;
	WHILE i <= CmdLength DO
		WHILE (CmdLine^[i] = ' ') AND (i <= CmdLength) DO
			INC(i);
			END;
		IF i <= CmdLength THEN
			start := i;
			WHILE (CmdLine^[i] # ' ') AND (i <= CmdLength) DO
				INC(i);
				END;
			ALLOCATE(Sptr, i - start + 2);
			j := 0;
			WHILE start <= i DO
				Sptr^[j] := CmdLine^[start];
				INC(start);
				INC(j);
				END;
			Sptr^[j] := nul;
			IF ArgC < MaxArgs THEN
				INC(ArgC);
				ArgV[ArgC] := Sptr;
				END;
			END;
		END;
	i := ArgC + 1;
	LOOP
		IF i > MaxArgs THEN EXIT END;
		ArgV[i] := NIL;
		INC(i);
		END;
END ParseCmdLine;

PROCEDURE GetEnv(EnvVar : ARRAY OF CHAR; VAR Strg : ARRAY OF CHAR);
CONST
	StringSize = 40;
VAR
	i, j, len : CARDINAL;
	Str1, Str2 : ARRAY[0..StringSize-1] OF CHAR;
BEGIN
	len := Length(EnvVar);
	FOR i := 0 TO (len - 1) DO
		IF (EnvVar[i] >= 'a') AND (EnvVar[i] <= 'z') THEN
			Str1[i] := CHR(ORD(EnvVar[i]) - ORD('a') + ORD('A'));
		ELSE
			Str1[i] := EnvVar[i];
			END;
		END;
	Str1[i + 1] := nul;
	i := 0;
	LOOP
		j := 0;
		WHILE (EnvPTR^[i] # '=') AND (EnvPTR^[i] # nul) DO
			Str2[j] := EnvPTR^[i];
			INC(i);
			INC(j);
			END;
		Str2[j] := nul;
		INC(i);
		IF CompareStr(Str1, Str2) = 0 THEN
			j := 0;
			WHILE EnvPTR^[i] # nul DO
				Strg[j] := EnvPTR^[i];
				INC(i);
				INC(j);
				END;
			Strg[j] := nul;
			EXIT;
		ELSE
			WHILE EnvPTR^[i] # nul DO
				INC(i);
				END;
			END;
		INC(i);
		IF EnvPTR^[i] = nul THEN
			Strg[0] := nul;
			EXIT;
			END;
		END;
END GetEnv;

BEGIN

(* Get PSP *)

SETREG(AX, 026H);
SWI(RTSVECTOR);
GETREG(BX, PSP.OFFSET);
GETREG(CX, PSP.SEGMENT);

ParseCmdLine;

(* Set up environment pointer *)

AnAddress := PSP;
INC(AnAddress, EnvOffSet);
EnvAdrSegment := AnAddress;
AnAddress.SEGMENT := EnvAdrSegment^;
AnAddress.OFFSET := 0;
EnvPTR := AnAddress;
END CmdEnv.