IMPLEMENTATION MODULE MakeForest;

  (*
   * MAKEMAKE.  Create a MAKEFILE for a MODULA-2 program.
   *
   * Written by Steve Tynor, 30 September 1986.
   *            UUCP  : tynor@gitpyr
   *            USNAIL: 2550 Akers Mill Rd. T-2, Atlanta GA. 30339
   *
   * Permission is granted to distribute, copy and change this program as long
   * as this notice remains...
   *)

  (*
   * Modified by R.A. Dukelow (24 January 1987) for to run under Logitech
   * Modula-2/86. See MakeMake.MOD for additional comments.
   *)

(*
                  ---------
 FileForst ----->| zzz.MOD |--> import list
                  ---------
                      |
                      .
                      .
                      .
                      V
                  ---------     ----------            ----------
                 | xxx.MOD |-->| import N |-->...--> | import 1 |
                  ---------     ----------            ----------
                      |
                  ---------
                 | xxx.DEF |--> import list
                  ---------
                      |
                  ----------
 FileForestEnd-->| main.MOD |--> import list
                  ----------

 Each element of an import list points back to a file descriptor (which may
 contain a ## comment ## rather than a real file name).
*)

IMPORT Strings, CmdEnv;
FROM   Storage IMPORT ALLOCATE;

  (*----------------------------------------------------------------------*)
  PROCEDURE FindFile (    fname          : ARRAY OF CHAR; 
                      VAR fileDefinition : FileDefinitionList);
  VAR
    found : BOOLEAN;
  BEGIN
    fileDefinition := FileForest;
    found := FALSE;
    WHILE (fileDefinition <> NIL) AND (NOT found) DO
      found := Strings.CompareStr (fname, fileDefinition^.filename) 
                            = 0 (* Strings.Equal *);
      IF NOT found THEN
        fileDefinition := fileDefinition^.next;
      END; (* IF *)
    END; (* WHILE *)
  END FindFile;


  (*----------------------------------------------------------------------*)
  PROCEDURE AddFile (    fname          : ARRAY OF CHAR; 
                     VAR fileDefinition : FileDefinitionList) : BOOLEAN;
  BEGIN
    FindFile (fname, fileDefinition);
    IF fileDefinition = NIL THEN
      NEW (fileDefinition);
      Strings.Assign (fname, fileDefinition^.filename);
      fileDefinition^.next    := FileForest;
      fileDefinition^.imports := NIL;

      (* until shown otherwise *)
      fileDefinition^.library := FALSE;
      fileDefinition^.local := TRUE;
      fileDefinition^.path := '';

      IF FileForest = NIL THEN
        FileForestEnd := fileDefinition;
      END; (* IF *)
      FileForest      := fileDefinition;
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END; (* IF *)
  END AddFile;


  (*----------------------------------------------------------------------*)
  PROCEDURE AddImport (fileDefinition :  FileDefinitionList;
                       fname          : ARRAY OF CHAR ); 
  VAR
    iPtr : ImportList;
  BEGIN
    NEW (iPtr);
    iPtr^.next              := fileDefinition^.imports;
    fileDefinition^.imports := iPtr;

    FindFile (fname, iPtr^.file);
    IF iPtr^.file = NIL THEN HALT END; (* file should already be exist *)
  END AddImport;

PROCEDURE InitSearchPaths(VAR SearchPaths: SearchPathList;
                          EnvName: ARRAY OF CHAR);
VAR
  str: String;
  i, j: CARDINAL;
  TmpPtr, TmpList, NewPathPtr: SearchPathList;
BEGIN
  (* build a temporary list in wrong order - reverse it later *)
  SearchPaths := NIL;
  TmpList := NIL;
  CmdEnv.GetEnv(EnvName, str);
  i := 0;
  WHILE (i <= HIGH(str)) AND ((str[i] = ' ') OR (str[i] = ';')) DO INC(i) END;
  WHILE (i <= HIGH(str)) AND (str[i] <> 0C) DO
    NEW(NewPathPtr);
    NewPathPtr^.next := TmpList;
    TmpList := NewPathPtr;
    j := 0;
    REPEAT
      NewPathPtr^.path[j] := str[i];
      INC(i);
      INC(j);
    UNTIL (i > HIGH(str)) OR (str[i] = 0C) OR (str[i] = ' ') OR (str[i] = ';');
    IF NewPathPtr^.path[j - 1] <> '\' THEN
      NewPathPtr^.path[j] := '\';
      INC(j);
    END;
    NewPathPtr^.path[j] := 0C;
    WHILE (i <= HIGH(str)) AND ((str[i] = ' ') OR (str[i] = ';')) DO
      INC(i);
    END;
  END; (* WHILE *)

  (* reverse the order of the list while attaching to SearchPaths *)
  WHILE TmpList <> NIL DO
    TmpPtr := TmpList;
    TmpList := TmpPtr^.next;
    TmpPtr^.next := SearchPaths;
    SearchPaths := TmpPtr;
  END;
END InitSearchPaths;


BEGIN
  FileForest := NIL;
  FileForestEnd := NIL;

  InitSearchPaths(LnkSearchPaths, 'M2LNK');
  InitSearchPaths(SymSearchPaths, 'M2SYM');
END MakeForest.