IMPLEMENTATION MODULE MakeParse;

  (*
   * MAKEMAKE.  Create a MAKEFILE for a MODULA-2 program.
   *
   * Written by Steve Tynor, 30 September 1986.
   *            UUCP  : tynor@gitpyr
   *            USNAIL: 2550 Akers Mill Rd. T-2, Atlanta GA. 30339
   *
   * Permission is granted to distribute, copy and change this program as long
   * as this notice remains...
   *)

  (*
   * Modified by R.A. Dukelow (24 January 1987) for to run under Logitech
   * Modula-2/86. See MakeMake.MOD for additional comments.
   *)

IMPORT MakeToken;
IMPORT MakeForest;
IMPORT Strings;
IMPORT MakeLibraries;

  (*----------------------------------------------------------------------*)
  PROCEDURE ModulenameToFilename (    modulename : ARRAY OF CHAR;
                                      defOrMod   : FileType;
                                  VAR fname      : ARRAY OF CHAR);
  CONST
    EOS = 0C;
  VAR
    c : CARDINAL;
  BEGIN
    c := 0;
    WHILE (c < HIGH (modulename)) AND (c < 8) AND (modulename[c] <> EOS) DO
      fname[c] := modulename[c];
      INC (c);
    END; (* WHILE *)
    fname[c] := '.';
    IF defOrMod = def THEN
      fname[c+1] := 'D'; fname[c+2] := 'E'; fname[c+3] := 'F'; 
      fname[c+4] := EOS;
    ELSE
      fname[c+1] := 'M'; fname[c+2] := 'O'; fname[c+3] := 'D';
      fname[c+4] := EOS;
    END; (* IF *)
  END ModulenameToFilename;


  (*----------------------------------------------------------------------*)
  PROCEDURE TerminatingToken(token : ARRAY OF CHAR) : BOOLEAN;
  VAR
    temp : BOOLEAN;
  BEGIN
    (* had to split up due to compiler limitation... *)
    temp := (Strings.CompareStr (token, 'CONST')     = 0 (* Strings.Equal *)) OR
            (Strings.CompareStr (token, 'TYPE')      = 0 (* Strings.Equal *)) OR
            (Strings.CompareStr (token, 'VAR')       = 0 (* Strings.Equal *)) OR
            (Strings.CompareStr (token, 'MODULE')    = 0 (* Strings.Equal *));
    RETURN  temp OR
            (Strings.CompareStr (token, 'PROCEDURE') = 0 (* Strings.Equal *)) OR
            (Strings.CompareStr (token, 'BEGIN')     = 0 (* Strings.Equal *)) OR
            (Strings.CompareStr (token, 'END')       = 0 (* Strings.Equal *));
  END TerminatingToken;


  (*----------------------------------------------------------------------*)
  PROCEDURE ParseFROM (     fileDef : MakeForest.FileDefinitionList;
                        VAR eof     : BOOLEAN );
  VAR
    modulename, filename : ARRAY [0 .. 100] OF CHAR;
  BEGIN
    MakeToken.NextToken (modulename, eof);
    ParseModule (modulename, def, filename);

    (* AddImport follows ParseModule(..., def, ...) because this creates
       the needed filename (xxx.DEF) for the dependency list and creates
       a file record on the list with all necessary checking. *)
    MakeForest.AddImport (fileDef, filename);

    MakeToken.SkipTillSemicolon (eof);

    ParseModule (modulename, mod, filename);
  END ParseFROM;

  (*----------------------------------------------------------------------*)
  PROCEDURE ParseIMPORT (    fileDef : MakeForest.FileDefinitionList;
                         VAR eof     : BOOLEAN );
  VAR
    modulename, filename : ARRAY [0 .. 100] OF CHAR;
  BEGIN
    MakeToken.NextToken (modulename, eof);
    WHILE modulename[0] <> ';' DO
      ParseModule (modulename, def, filename);

    (* AddImport follows ParseModule(..., def, ...) because this creates
       the needed filename (xxx.DEF) for the dependency list and creates
       a file record on the list with all necessary checking. *)
      MakeForest.AddImport (fileDef, filename);

      ParseModule (modulename, mod, filename);
      MakeToken.NextToken (modulename, eof);
    END; (* WHILE *)
  END ParseIMPORT;

  (*----------------------------------------------------------------------*)
  PROCEDURE FindPath (VAR FileDefinition :  MakeForest.FileDefinitionList );
  (* Try to find a path to the indicated file *)
  VAR
    PartFilename, FullFilename: ARRAY[0..100] OF CHAR;
    success: BOOLEAN;
    PathPtr: MakeForest.SearchPathList;
    ColPeriod: CARDINAL;
  BEGIN
    (* strip off the extension *)
    ColPeriod := Strings.Pos('.', FileDefinition^.filename);
    Strings.Copy(FileDefinition^.filename, 0, ColPeriod, PartFilename);

    (* add extension and find appropriate search path list *)
    IF Strings.Pos('.MOD', FileDefinition^.filename) <=
       HIGH(FileDefinition^.filename) THEN
      PathPtr := MakeForest.LnkSearchPaths;
      Strings.Concat(PartFilename, '.LNK', PartFilename);
    ELSE
      PathPtr := MakeForest.SymSearchPaths;
      Strings.Concat(PartFilename, '.SYM', PartFilename);
    END;

    (* do the search *)
    success := FALSE;
    WHILE (NOT success) AND (PathPtr <> NIL) DO
      Strings.Concat(PathPtr^.path, PartFilename, FullFilename);
      MakeToken.OpenFile(FullFilename, success);
      IF NOT success THEN PathPtr := PathPtr^.next END;
      MakeToken.CloseFile;
    END;
    IF success THEN
      FileDefinition^.path := PathPtr^.path;
    ELSE
      FileDefinition^.path := '##UnknownPath##\';
    END;
  END FindPath; 

  (*----------------------------------------------------------------------*)
  PROCEDURE ParseModule (    modulename : ARRAY OF CHAR;
                             type       : FileType;
                         VAR filename   : ARRAY OF CHAR);
  VAR 
    token    : ARRAY [0 .. 132] OF CHAR;
    fileDef  : MakeForest.FileDefinitionList;
    success,
    eof      : BOOLEAN;

  BEGIN
    ModulenameToFilename (modulename, type, filename);

    (* add current filename to list if necessary *)
    IF MakeForest.AddFile (filename, fileDef) THEN
      (* this is a new file - map it out if can find it *)
      MakeToken.OpenFile (filename, success);

      IF NOT success THEN
        fileDef^.local := FALSE;
        IF MakeLibraries.IsALibraryModule (modulename) THEN
          fileDef^.library := TRUE;
        ELSE
          FindPath(fileDef);
        END; (* IF *)
      ELSE
        MakeToken.NextToken (token, eof);
        IF    (type = def) AND NOT
              (Strings.CompareStr (token, 'DEFINITION') = 0 (* Strings.Equal *)) THEN
          MakeForest.AddImport (fileDef, 
                                "## Expected a 'DEFINITION' module ##");
        ELSIF (type = mod) AND NOT
              (Strings.CompareStr (token, 'IMPLEMENTATION') = 0 (* Strings.Equal *)) THEN
          MakeForest.AddImport (fileDef, 
                                "## Expected an 'IMPLEMENTATION' module ##");
        ELSIF (type = main) AND NOT
              (Strings.CompareStr (token, 'MODULE') = 0 (* Strings.Equal *)) THEN
          MakeForest.AddImport (fileDef, 
                                "## Expected a 'MODULE' module ##");
        END; (* IF / ELSE / ENDIF *)
        MakeToken.SkipTillSemicolon (eof);
        token := ';';
        WHILE (NOT eof) AND (NOT TerminatingToken(token)) DO
          IF Strings.CompareStr (token, 'FROM') = 0 (* Strings.Equal *) THEN
            ParseFROM (fileDef, eof);
          ELSIF Strings.CompareStr (token, 'IMPORT') = 0 (* Strings.Equal *) THEN
            ParseIMPORT (fileDef, eof);
          END; (* IF *)
          MakeToken.NextToken (token, eof);
        END; (* WHILE *)
      END; (* IF file opened *)
      MakeToken.CloseFile;
    END; (* IF we need to check this one *)

  END ParseModule;

END MakeParse.