IMPLEMENTATION MODULE MakeToken;

  (*
   * MAKEMAKE.  Create a MAKEFILE for a MODULA-2 program.
   *
   * Written by Steve Tynor, 30 September 1986.
   *            UUCP  : tynor@gitpyr
   *            USNAIL: 2550 Akers Mill Rd. T-2, Atlanta GA. 30339
   *
   * Permission is granted to distribute, copy and change this program as long
   * as this notice remains...
   *)

  (*
   * Modified by R.A. Dukelow (24 January 1987) for to run under Logitech
   * Modula-2/86. See MakeMake.MOD for additional comments.
   *)

FROM SYSTEM IMPORT BYTE;
IMPORT FileSystem;

VAR 
  currentCh : CHAR;
  currentStream : FileSystem.File;
  
  (*======================================================================*)
  MODULE StreamStack;
  IMPORT FileSystem;
  EXPORT Pop, Push;

  CONST
    StackSize = 20;
  VAR
    StrStack  : ARRAY [1 .. StackSize] OF FileSystem.File;
    CharStack : ARRAY [1 .. StackSize] OF CHAR;
    StackPtr  : CARDINAL;

    (*--------------------------------------------------------------------*)
    PROCEDURE Push (str : FileSystem.File; ch : CHAR);
    BEGIN
      IF StackPtr + 1 <= StackSize THEN
        INC (StackPtr);
        StrStack [StackPtr] := str;
        CharStack[StackPtr] := ch;
      END; (* IF *)
    END Push;

    (*--------------------------------------------------------------------*)
    PROCEDURE Pop (VAR str : FileSystem.File; VAR ch : CHAR);
    BEGIN
      IF StackPtr > 0 THEN
        str := StrStack [StackPtr];
        ch  := CharStack[StackPtr];
        DEC (StackPtr);
      END; (* IF *)
    END Pop;

  BEGIN
    StackPtr := 0;
  END StreamStack;
  (*======================================================================*)
 

  (*----------------------------------------------------------------------*)
  PROCEDURE OpenFile (    filename : ARRAY OF CHAR;
                      VAR success  : BOOLEAN);
  BEGIN
    Push (currentStream, currentCh);
    (* open file for reading only *)
    FileSystem.Lookup(currentStream, filename, FALSE);
    currentCh := ' ';
    success := currentStream.res = FileSystem.done;
  END OpenFile;

  (*----------------------------------------------------------------------*)
  PROCEDURE CloseFile;
  BEGIN
    FileSystem.Close(currentStream);
    Pop (currentStream, currentCh);
  END CloseFile;


  (*----------------------------------------------------------------------*)
  PROCEDURE SeparatingChar (ch : CHAR) : BOOLEAN;
    (* we're just worried about a subset of the real separating chars... *)
  BEGIN
    RETURN NOT (((ORD(ch) >= ORD('a')) AND (ORD(ch) <= ORD('z'))) OR
                ((ORD(ch) >= ORD('A')) AND (ORD(ch) <= ORD('Z'))) OR
                ((ORD(ch) >= ORD('0')) AND (ORD(ch) <= ORD('9'))));
  END SeparatingChar;


  (*----------------------------------------------------------------------*)
  PROCEDURE NextToken (VAR token : ARRAY OF CHAR;
                       VAR eof   : BOOLEAN);
  VAR
    c : CARDINAL;
    b: BYTE;
  BEGIN
    c := 0;
    IF currentCh = ';' THEN
      token[0] := ';';
      token[1] := 0C;
      eof := currentStream.eof;
      FileSystem.ReadByte(currentStream, b);
      currentCh := CHAR(b);
      RETURN;
    END; (* IF *)
    WHILE SeparatingChar (currentCh) DO
      FileSystem.ReadByte(currentStream, b);
      currentCh := CHAR(b);
      IF currentCh = ';' THEN
        token[0] := ';';
        token[1] := 0C;
        eof := currentStream.eof;
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        RETURN;
      ELSIF currentCh = '(' THEN
        token[c] := currentCh;
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        IF currentCh = '*' THEN
          SkipComment;
        ELSE
          token[0] := '(';
          token[1] := currentCh;
          c := 2;
        END; (* IF *)
      END; (* IF *)
    END; (* WHILE *)
    REPEAT
      IF currentCh = '(' THEN
        token[c] := currentCh;
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        IF currentCh = '*' THEN
          IF c > 0 THEN
            token[c+1] := 0C;
            SkipComment;
            RETURN;
          ELSE
            SkipComment;
          END; (* IF *)
        END; (* IF *)
      END; (* IF *)
      token[c] := currentCh;
      INC (c);
      FileSystem.ReadByte(currentStream, b);
      currentCh := CHAR(b);
    UNTIL currentStream.eof OR 
          (c > HIGH (token)) OR 
          SeparatingChar (currentCh);
    token[c] := 0C;
    eof := currentStream.eof;
  END NextToken;


  (*----------------------------------------------------------------------*)
  PROCEDURE SkipTillSemicolon (VAR eof   : BOOLEAN);
  VAR
    ch : CHAR;
    b: BYTE;
  BEGIN
    REPEAT
      IF (NOT currentStream.eof) AND (currentCh = '(') THEN
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        IF (NOT currentStream.eof) AND (currentCh = '*') THEN
          SkipComment;
        END; (* IF *)
      ELSE
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
      END; (* IF *)
    UNTIL currentStream.eof OR (currentCh = ';');
  END SkipTillSemicolon;


  (*----------------------------------------------------------------------*)
  PROCEDURE SkipComment;
  VAR
    level : CARDINAL;
    done  : BOOLEAN;
    b: BYTE;
  BEGIN
    level := 1;
    FileSystem.ReadByte(currentStream, b);
    currentCh := CHAR(b);
    done := FALSE;
    REPEAT
      IF (NOT currentStream.eof) AND (currentCh = '*') THEN
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        IF (NOT currentStream.eof) AND (currentCh = ')') THEN
          DEC(level);
          done := level = 0;
        END; (* IF *)
      ELSIF (NOT currentStream.eof) AND (currentCh = '(') THEN
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
        IF (NOT currentStream.eof) AND (currentCh = '*') THEN
          INC(level);
        END; (* IF *)
      ELSE
        FileSystem.ReadByte(currentStream, b);
        currentCh := CHAR(b);
      END; (* IF *)
    UNTIL currentStream.eof OR done;
  END SkipComment;

BEGIN
  currentCh := ' ';
END MakeToken.